@@ -12,7 +12,6 @@
 #include "interval.h"
 #include "rule_p.h"
 #include "logging.h"
-#include "consecutiveaccumulator_p.h"
 
 #include <QDateTime>
 #include <QJsonArray>
@@ -81,6 +80,13 @@ void OpeningHoursPrivate::autocorrect()
             // the current rule only has a time selector, so we append that to the previous rule
             else if (curRuleSingleSelector && rule->m_timeSelector && prevRule->m_timeSelector) {
                 appendSelector(prevRule->m_timeSelector.get(), std::move(rule->m_timeSelector));
+                prevRule->copyStateFrom(*rule);
+                it = std::prev(m_rules.erase(it));
+            }
+
+            // previous is a single weekday selector and current is a single time selector
+            else if (curRuleSingleSelector && prevRuleSingleSelector && rule->m_timeSelector && prevRule->m_weekdaySelector) {
+                prevRule->m_timeSelector = std::move(rule->m_timeSelector);
                 it = std::prev(m_rules.erase(it));
             }
 
@@ -93,12 +99,22 @@ void OpeningHoursPrivate::autocorrect()
                 std::swap(*it, *std::prev(it));
                 it = std::prev(m_rules.erase(it));
             }
+
+            // previous has no time selector and the current one is a misplaced 24/7 rule:
+            // convert the 24/7 to a 00:00-24:00 time selector
+            else if (rule->selectorCount() == 0 && rule->m_seen_24_7 && !prevRule->m_timeSelector) {
+                prevRule->m_timeSelector.reset(new Timespan);
+                prevRule->m_timeSelector->begin = { Time::NoEvent, 0, 0 };
+                prevRule->m_timeSelector->end = { Time::NoEvent, 24, 0 };
+                it = std::prev(m_rules.erase(it));
+            }
         } else if (rule->m_ruleType == Rule::NormalRule) {
             // Previous rule has time and other selectors
             // Current rule is only a time selector
             // "Mo-Sa 12:00-15:00; 18:00-24:00" => "Mo-Sa 12:00-15:00,18:00-24:00"
             if (curRuleSingleSelector && rule->m_timeSelector
-                    && prevRule->selectorCount() > 1 && prevRule->m_timeSelector) {
+                    && prevRule->selectorCount() > 1 && prevRule->m_timeSelector
+                    && rule->state() == prevRule->state()) {
                 appendSelector(prevRule->m_timeSelector.get(), std::move(rule->m_timeSelector));
                 it = std::prev(m_rules.erase(it));
             }
@@ -113,6 +129,7 @@ void OpeningHoursPrivate::autocorrect()
                      && rule->selectorCount() == 2 && rule->m_weekdaySelector && prevRule->m_weekdaySelector
                      // slower than writing an operator==, but so much easier to write :)
                      && rule->m_weekdaySelector->toExpression() == prevRule->m_weekdaySelector->toExpression()
+                     && rule->state() == prevRule->state()
                      ) {
                 appendSelector(prevRule->m_timeSelector.get(), std::move(rule->m_timeSelector));
                 it = std::prev(m_rules.erase(it));
@@ -176,6 +193,9 @@ void OpeningHoursPrivate::simplify()
         if (rule->m_weekdaySelector) {
             rule->m_weekdaySelector->simplify();
         }
+        if (rule->m_monthdaySelector) {
+            rule->m_monthdaySelector->simplify();
+        }
     }
 }
 
@@ -411,8 +431,9 @@ QByteArray OpeningHours::normalizedExpression() const
 
 QByteArray OpeningHours::simplifiedExpression() const
 {
-    d->simplify();
-    return normalizedExpression();
+    OpeningHours copy(normalizedExpression());
+    copy.d->simplify();
+    return copy.normalizedExpression();
 }
 
 QString OpeningHours::normalizedExpressionString() const
